#ifndef CORE_TEST_H
#define CORE_TEST_H

#include "core/Logger.hpp"
#include "core/Types.hpp"

void finalizeTests(void);
bool addToResult(const char* file, bool comparison);

template<typename T>
bool testEqual(const char* file, int line, const T& wanted, const T& actual) {
    file = getShortFileName(file);
    if(addToResult(file, wanted == actual)) {
        return true;
    }
    char buffer[512];
    formatBuffer(
        buffer, sizeof(buffer),
        TERMINAL_RED "#:# - expected '#' got '#'" TERMINAL_RESET, file, line,
        wanted, actual);
    puts(buffer);
    return false;
}

#define TEST_ARGS const char *file, int line

bool testString(TEST_ARGS, const char* wanted, const char* actual);
bool testFloat(TEST_ARGS, float wanted, float actual, float error);
bool testNull(TEST_ARGS, const void* p);
bool testNotNull(TEST_ARGS, const void* p);

#define TEST_FLOAT(wanted, actual, error)                \
    testFloat(__FILE__, __LINE__, wanted, actual, error)

#define TEST(wanted, actual)                                        \
    testEqual<decltype(wanted)>(__FILE__, __LINE__, wanted, actual)
#define TEST_STRING(wanted, actual)                \
    testString(__FILE__, __LINE__, wanted, actual)
#define TEST_FALSE(actual) TEST(false, actual)
#define TEST_TRUE(actual) TEST(true, actual)
#define TEST_NULL(actual) testNull(__FILE__, __LINE__, actual)
#define TEST_NOT_NULL(actual) testNotNull(__FILE__, __LINE__, actual)

#endif
