#include "core/File.hpp"

#include <stdio.h>

#include "ErrorSimulator.hpp"
#include "core/Logger.hpp"
#include "core/Utility.hpp"

static bool readOpenFile(FILE* file, FileContent* f, const char* path) {
    if(FAIL_STEP || fseek(file, 0, SEEK_END)) {
        REPORT(LogLevel::ERROR, "cannot seek file end of '#'", path);
        return true;
    }
    long l = ftell(file);
    if(FAIL_STEP || l < 0) {
        REPORT(LogLevel::ERROR, "cannot tell file position of '#'", path);
        return true;
    }
    f->length = (size_t)l;
    if(FAIL_STEP || fseek(file, 0, SEEK_SET)) {
        REPORT(LogLevel::ERROR, "cannot seek file start of '#'", path);
        return true;
    }
    f->data = coreNewN(char, f->length + 1);
    size_t read = fread(f->data, 1, f->length, file);
    f->data[f->length] = 0;
    if(FAIL_STEP || read != f->length) {
        REPORT(
            LogLevel::ERROR, "expected to read # bytes from '#' but read #",
            f->length, path, read);
        return true;
    }
    return false;
}

bool readFile(FileContent* f, const char* path) {
    FILE* file = fopen(path, "rb");
    if(file == nullptr) {
        REPORT(LogLevel::ERROR, "cannot read file '#'", path);
        return true;
    }
    bool r = readOpenFile(file, f, path);
    if(FAIL_STEP || fclose(file)) {
        REPORT(LogLevel::ERROR, "cannot close file '#'", path);
        r = true;
    }
    if(r) {
        destroyFileContent(f);
    }
    return r;
}

void destroyFileContent(FileContent* f) {
    coreDeleteN(f->data);
    *f = {};
}
